#ifdef __HUMAN68K__
#include <string.h>
#include <sys/xstart.h>
#include "make.h"

typedef struct indirect {
	struct indirect	*next;
	char	*command;
	char	*prefix;
} indirect;

static indirect	*indirects;

void construct_indirect_list( char *command, char *prefix )
{
	if ( command ) {
		indirect	*ind = (indirect *)xmalloc( sizeof( indirect ));
		ind->next = indirects;
		ind->command = savestring( command, strlen( command ));
		if ( prefix )
			ind->prefix = savestring( prefix, strlen( prefix ));
		else
			ind->prefix = savestring( "", 0 );
		indirects = ind;
	}
}

void build_indirect_lists( void )
{
	static char	*default_indirect_rules[] = {
					"lk" , "-i",
					"ar" , "-i",
					"lha", "@" ,
					0
				};
	char	**p = &default_indirect_rules[0];

	if ( no_builtin_rules_flag )
		return;

	while( *p ) {
		char	*command = *p++;
		char	*prefix = *p++;

		construct_indirect_list( command, prefix );
	}

	{
		indirect *new = 0;
		indirect *old, *nexto;

		/* Xgt] */
		for (old = indirects; old != 0; old = nexto) {
			nexto = old->next;
			old->next = new;
			new = old;
		}
		indirects = new;
	}
}

void print_indirect_data_base ( void )
{
	unsigned int	ninds;
	indirect	*ind;

	puts("\n# `indirect' flag list\n");

	ninds = 0;
	for ( ind = indirects; ind != 0; ind = ind->next ) {
		ninds++;
		printf ( "indirect %s %s\n", ind->command, ind->prefix );
	}

	if ( ninds == 0 )
		puts( "# No `indirect' flag." );
	else
		printf( "\n# %u `indirect' flags.", ninds );
}

static char *get_indirect_prefix( char *cmdline )
{
	char	*ret = (char *)0;
	indirect	*ind;

	if (( ind = indirects ) != 0 ) {
		char	*buf;
		buf = xmalloc( strlen( cmdline ));
		_dehupair( cmdline, buf );
		do {
			if ( !strcmp( ind->command, buf )) {
				ret = ind->prefix;
				break;
			}
			ind = ind->next;
		} while ( ind );
		free( buf );
	}
	return ret;
}

static char **chgenv_and_setenv( char **envp, char *string )
{
	char	**ret, **saved_environ;

	saved_environ = environ;
	environ = envp;				/* environ ޔ */
	if ( putenv( string ))
		fatal (" virtual memory exhausted" );	/* s */
	ret = environ;	/* environ Ċmۂꂽꍇ͕K envp ƈvȂ */
	environ = saved_environ;	/* environ A */
	return ret;
}

/*========================================================================
	ϐ HUGEARG 폜
========================================================================*/
char **removeenv_for_indirect( char **envp )
{
	return chgenv_and_setenv( envp, "HUGEARG=" );
}

/*========================================================================
	ϐ HUGEARG ݒ
========================================================================*/
char **setenv_for_indirect( char **envp, char *cmdline )
{
	char	**ret;
	char	*prefix, *buf;

	prefix = get_indirect_prefix( cmdline );
	if ( prefix ) {
		char	**old_environ,
				hugearg_str[] = "HUGEARG=indirect ";

		buf = xmalloc( sizeof( hugearg_str ) + strlen( prefix ) + 1 );
		strcpy( buf, hugearg_str );
		strcat( buf, prefix );	/* buf = "HUGEARG=indirect xxx" */
		ret = chgenv_and_setenv( envp, buf );
		free( buf );
	} else {
		ret = removeenv_for_indirect( envp );	/* Ô */
	}
	return ret;
}

#endif /* __HUMAN68K__ */
